@@ -3,8 +3,9 @@ This is a template script - an example of how a CEA script should be set up.
 
 NOTE: ADD YOUR SCRIPT'S DOCUMENTATION HERE (what, why, include literature references)
 """
-from __future__ import division
-from __future__ import print_function
+
+
+
 
 import os
 import cea.config
@@ -32,14 +33,12 @@ def main(config):
     """
     This is the main entry point to your script. Any parameters used by your script must be present in the ``config``
     parameter. The CLI will call this ``main`` function passing in a ``config`` object after adjusting the configuration
-    to reflect parameters passed on the command line - this is how the ArcGIS interface interacts with the scripts
-    BTW.
+    to reflect parameters passed on the command line / user interface
 
     :param config:
     :type config: cea.config.Configuration
     :return:
     """
-    assert os.path.exists(config.scenario), 'Scenario not found: %s' % config.scenario
     locator = cea.inputlocator.InputLocator(config.scenario)
 
     template(locator, config.scenario)
