@@ -2,31 +2,35 @@
 Photovoltaic thermal panels
 """
 
-from __future__ import division
-from __future__ import print_function
+
+
+
 
 import os
 import time
+from itertools import repeat
 from math import *
-from numba import jit
+
 import geopandas as gpd
 import numpy as np
 import pandas as pd
 from geopandas import GeoDataFrame as gdf
-from itertools import izip, repeat
-import cea.utilities.workerstream
+from numba import jit
+
 import cea.inputlocator
+import cea.utilities.parallel
+import cea.utilities.workerstream
+from cea.constants import HOURS_IN_YEAR
+from cea.technologies.solar import constants
 from cea.technologies.solar.photovoltaic import (calc_properties_PV_db, calc_PV_power, calc_diffuseground_comp,
                                                  calc_absorbed_radiation_PV, calc_cell_temperature)
 from cea.technologies.solar.solar_collector import (calc_properties_SC_db, calc_IAM_beam_SC, calc_q_rad, calc_q_gain,
                                                     vectorize_calc_Eaux_SC, calc_optimal_mass_flow,
                                                     calc_optimal_mass_flow_2, calc_qloss_network)
-from cea.technologies.solar import constants
 from cea.utilities import epwreader
 from cea.utilities import solar_equations
 from cea.utilities.standardize_coordinates import get_lat_lon_projected_shapefile
-from cea.constants import HOURS_IN_YEAR
-import cea.utilities.parallel
+from cea.analysis.costs.equations import calc_capex_annualized, calc_opex_annualized
 
 __author__ = "Jimeno A. Fonseca"
 __copyright__ = "Copyright 2015, Architecture and Building Systems - ETH Zurich"
@@ -62,9 +66,9 @@ def calc_PVT(locator, config, latitude, longitude, weather_data, date_local, bui
     :return: Building_PVT.csv with solar collectors heat generation potential of each building, Building_PVT_sensors.csv
              with sensor data of each PVT panel.
     """
-    t0 = time.clock()
+    t0 = time.perf_counter()
 
-    radiation_json_path = locator.get_radiation_building(building_name)
+    radiation_json_path = locator.get_radiation_building_sensors(building_name)
     metadata_csv_path = locator.get_radiation_metadata(building_name)
 
     # solar properties
@@ -72,8 +76,8 @@ def calc_PVT(locator, config, latitude, longitude, weather_data, date_local, bui
     print('calculating solar properties done for building %s' % building_name)
 
     # get properties of the panel to evaluate # TODO: find a PVT module reference
-    panel_properties_PV = calc_properties_PV_db(locator.get_supply_systems(), config)
-    panel_properties_SC = calc_properties_SC_db(locator.get_supply_systems(), config)
+    panel_properties_PV = calc_properties_PV_db(locator.get_database_conversion_systems(), config)
+    panel_properties_SC = calc_properties_SC_db(locator.get_database_conversion_systems(), config)
     print('gathering properties of PVT collector panel for building %s' % building_name)
 
     # select sensor point with sufficient solar radiation
@@ -102,16 +106,16 @@ def calc_PVT(locator, config, latitude, longitude, weather_data, date_local, bui
         Final = calc_PVT_generation(sensor_groups, weather_data, date_local, solar_properties, latitude,
                                     tot_bui_height_m, panel_properties_SC, panel_properties_PV, config)
 
-        Final.to_csv(locator.PVT_results(building_name=building_name), index=True, float_format='%.2f')
-        sensors_metadata_cat.to_csv(locator.PVT_metadata_results(building_name=building_name), index=True,
+        Final.to_csv(locator.PVT_results(building=building_name), index=True, float_format='%.2f')
+        sensors_metadata_cat.to_csv(locator.PVT_metadata_results(building=building_name), index=True,
                                     index_label='SURFACE',
                                     float_format='%.2f')  # print selected metadata of the selected sensors
 
-        print('Building', building_name, 'done - time elapsed:', (time.clock() - t0), ' seconds')
+        print('Building', building_name, 'done - time elapsed:', (time.perf_counter() - t0), ' seconds')
 
     else:  # This block is activated when a building has not sufficient solar potential
         Final = pd.DataFrame(
-            {'PVT_walls_north_E_kWh': 0.0, 'PVT_walls_north_m2': 0.0, 'PVT_walls_north_Q_kWh': 0.0,
+            {'Date': date_local, 'PVT_walls_north_E_kWh': 0.0, 'PVT_walls_north_m2': 0.0, 'PVT_walls_north_Q_kWh': 0.0,
              'PVT_walls_north_Tout_C': 0.0,
              'PVT_walls_south_E_kWh': 0.0, 'PVT_walls_south_m2': 0, 'PVT_walls_south_Q_kWh': 0.0,
              'PVT_walls_south_Tout_C': 0.0,
@@ -125,13 +129,13 @@ def calc_PVT(locator, config, latitude, longitude, weather_data, date_local, bui
              'mcp_PVT_kWperC': 0.0, 'Eaux_PVT_kWh': 0.0,
              'Q_PVT_l_kWh': 0.0, 'E_PVT_gen_kWh': 0.0, 'Area_PVT_m2': 0.0,
              'radiation_kWh': 0.0}, index=range(HOURS_IN_YEAR))
-        Final.to_csv(locator.PVT_results(building_name=building_name), index=True, float_format='%.2f', na_rep='nan')
+        Final.to_csv(locator.PVT_results(building=building_name), index=False, float_format='%.2f', na_rep='nan')
         sensors_metadata_cat = pd.DataFrame(
             {'SURFACE': 0, 'AREA_m2': 0, 'BUILDING': 0, 'TYPE': 0, 'Xcoor': 0, 'Xdir': 0, 'Ycoor': 0, 'Ydir': 0,
              'Zcoor': 0, 'Zdir': 0, 'orientation': 0, 'total_rad_Whm2': 0, 'tilt_deg': 0, 'B_deg': 0,
              'array_spacing_m': 0, 'surface_azimuth_deg': 0, 'area_installed_module_m2': 0,
              'CATteta_z': 0, 'CATB': 0, 'CATGB': 0, 'type_orientation': 0}, index=range(2))
-        sensors_metadata_cat.to_csv(locator.PVT_metadata_results(building_name=building_name), index=True,
+        sensors_metadata_cat.to_csv(locator.PVT_metadata_results(building=building_name), index=False,
                                     float_format='%.2f')
 
     return
@@ -184,7 +188,7 @@ def calc_PVT_generation(sensor_groups, weather_data, date_local, solar_propertie
 
     list_results_from_PVT = list(range(number_groups))
 
-    potential = pd.DataFrame(index=[range(HOURS_IN_YEAR)])
+    potential = pd.DataFrame(index=range(HOURS_IN_YEAR))
     panel_orientations = ['walls_south', 'walls_north', 'roofs_top', 'walls_east', 'walls_west']
     for panel_orientation in panel_orientations:
         potential['PVT_' + panel_orientation + '_Q_kWh'] = 0.0
@@ -621,35 +625,35 @@ def calc_Mfl_kgpers(DELT, Nseg, STORED, TIME0, Tin_C, specific_flows_kgpers, tim
 
 # investment and maintenance costs
 
-def calc_Cinv_PVT(PVT_peak_kW, locator, config, technology=0):
+def calc_Cinv_PVT(PVT_peak_W, locator, technology=0):
     """
     P_peak in kW
     result in CHF
     technology = 0 represents the first technology when there are multiple technologies.
     FIXME: handle multiple technologies when cost calculations are done
     """
-    if PVT_peak_kW > 0.0:
-        PVT_peak_W = PVT_peak_kW * 1000  # converting to W from kW
-        PVT_cost_data = pd.read_excel(locator.get_supply_systems(), sheet_name="PV")
+    if PVT_peak_W > 0.0:
+        PVT_cost_data = pd.read_excel(locator.get_database_conversion_systems(), sheet_name="PV")
         technology_code = list(set(PVT_cost_data['code']))
-        PVT_cost_data[PVT_cost_data['code'] == technology_code[technology]]
+        PVT_cost_data = PVT_cost_data[PVT_cost_data['code'] == technology_code[technology]]
         # if the Q_design is below the lowest capacity available for the technology, then it is replaced by the least
         # capacity for the corresponding technology from the database
-        if PVT_peak_W < PVT_cost_data['cap_min'][0]:
-            PVT_peak_W = PVT_cost_data['cap_min'][0]
-        PVT_cost_data = PVT_cost_data[(PVT_cost_data['cap_min'] <= PVT_peak_W) & (PVT_cost_data['cap_max'] > PVT_peak_W)]
+        if PVT_peak_W < PVT_cost_data['cap_min'].values[0]:
+            PVT_peak_W = PVT_cost_data['cap_min'].values[0]
+        PVT_cost_data = PVT_cost_data[
+            (PVT_cost_data['cap_min'] <= PVT_peak_W) & (PVT_cost_data['cap_max'] > PVT_peak_W)]
         Inv_a = PVT_cost_data.iloc[0]['a']
         Inv_b = PVT_cost_data.iloc[0]['b']
         Inv_c = PVT_cost_data.iloc[0]['c']
         Inv_d = PVT_cost_data.iloc[0]['d']
         Inv_e = PVT_cost_data.iloc[0]['e']
-        Inv_IR = (PVT_cost_data.iloc[0]['IR_%']) / 100
+        Inv_IR = PVT_cost_data.iloc[0]['IR_%']
         Inv_LT = PVT_cost_data.iloc[0]['LT_yr']
         Inv_OM = PVT_cost_data.iloc[0]['O&M_%'] / 100
 
         InvC = Inv_a + Inv_b * (PVT_peak_W) ** Inv_c + (Inv_d + Inv_e * PVT_peak_W) * log(PVT_peak_W)
 
-        Capex_a = InvC * (Inv_IR) * (1 + Inv_IR) ** Inv_LT / ((1 + Inv_IR) ** Inv_LT - 1)
+        Capex_a = calc_capex_annualized(InvC, Inv_IR, Inv_LT)
         Opex_fixed = InvC * Inv_OM
         Capex = InvC
     else:
@@ -672,8 +676,6 @@ def main(config):
     print('Running photovoltaic-thermal with type-pvpanel = %s' % config.solar.type_pvpanel)
 
     building_names = config.solar.buildings
-    if not building_names:
-        building_names = locator.get_zone_building_names()
 
     hourly_results_per_building = gdf.from_file(locator.get_zone_geometry())
     latitude, longitude = get_lat_lon_projected_shapefile(hourly_results_per_building)
@@ -721,7 +723,7 @@ def main(config):
     aggregated_hourly_results_df.to_csv(locator.PVT_totals(), index=True, float_format='%.2f', na_rep='nan')
     # save annual results
     aggregated_annual_results_df = pd.DataFrame(aggregated_annual_results).T
-    aggregated_annual_results_df.to_csv(locator.PVT_total_buildings(), index=True, float_format='%.2f')
+    aggregated_annual_results_df.to_csv(locator.PVT_total_buildings(), index=True, index_label="Name", float_format='%.2f')
 
 
 if __name__ == '__main__':
